
# q-expansion of newform 3332.2.a.r, downloaded from the LMFDB on 10 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3332
weight = 2
poly_data = [12, 3, -9, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-5, -2, 1, 0], 1], [[6, -5, -3, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0], [-1, 1, 0, 0], [0, 1, 0, 0], [0, 0, 0, 0], [3, 0, 0, -1], [0, -1, 1, -1], [1, 0, 0, 0], [-1, 0, -1, -1], [0, -2, 0, -2], [1, -2, -1, 1], [-3, 2, 1, 0], [2, 1, 0, 1], [5, 1, 1, -2], [3, 0, -1, 1], [-5, 2, -1, 1], [-3, 0, 0, -1], [-5, 1, -1, 1], [2, -1, 2, -2], [0, -1, 2, -2], [3, -2, 0, -4], [-7, -1, -1, -1], [-4, 1, -3, 2], [7, -1, -1, -2], [2, 2, 1, 1], [4, -3, 0, -1], [-5, -1, -1, 1], [7, -2, 3, -2], [4, 3, -1, 0], [1, 0, 1, 2], [12, 1, 0, 0], [-5, 0, -5, 2], [1, 0, -1, 3], [6, 0, -3, 2], [-5, -1, 0, 3], [3, 1, 0, -1], [-5, -4, 1, 2], [12, -4, 1, 0], [7, -3, 1, -1], [-13, 1, -2, 3], [2, -1, -2, 1], [0, -5, 0, -2], [7, -2, 3, 3], [-10, 4, -2, -2], [13, 1, 1, -3], [6, 1, 0, 0], [9, -1, 4, -3], [-13, 3, -3, -1], [-1, -2, -1, -3], [2, 0, 1, -1], [14, -4, 2, -1], [-2, -2, 2, 0], [-13, 5, 1, 0], [-1, -3, -1, 4], [-8, 5, 2, 3], [-9, 0, -6, 0], [-6, -2, 0, 1], [-2, -8, 2, -5], [11, -1, -1, -2], [5, -2, -3, 2], [1, 9, -4, 6], [5, 2, -4, 2], [17, -2, -2, -2], [-10, 3, 2, -2], [-9, 2, 0, -3], [7, -5, 3, 3], [8, 4, 4, -3], [18, 1, 2, -1], [10, -3, 4, -1], [6, -2, 6, -2], [-8, -3, 0, 3], [-5, -1, 2, 0], [-9, 8, 3, -1], [0, 5, 1, 7], [8, 6, -3, 0], [10, -4, 7, -1], [2, 4, -2, -3], [-8, -1, -1, 2], [10, 3, 6, 3], [-7, 0, -5, 6], [9, -5, 4, -10], [-6, -6, 3, -6], [6, 4, 2, -7], [2, -11, 1, 2], [-2, 0, 0, 6], [-14, -2, -3, 2], [12, -6, 0, -9], [-12, 4, 6, -4], [29, -1, 3, -2], [-1, 3, -2, 2], [-7, 8, 3, -1], [8, -3, 4, -6], [17, -7, -5, 1], [9, 3, 8, -7], [-5, -8, -1, 3], [-3, -6, 2, 4], [14, -5, -5, 2], [-19, -1, -5, -4], [-20, 3, 2, 1], [-1, 3, -1, 1], [2, 0, 0, -6], [-1, 7, -6, 7], [-5, -2, 2, 7], [8, -9, 4, 1], [-7, 0, 4, -2], [6, 1, 2, 2], [7, 5, 0, -4], [18, 3, 6, -3], [-2, 0, 5, 3], [-10, -4, 4, -2], [0, -5, -2, -8], [-3, 0, -8, 7], [4, 1, 1, 3], [8, 0, -2, 5], [-6, -7, 1, 6], [-9, 6, 5, 2], [15, 2, 3, -1], [-11, -8, 0, -4], [-12, 1, 0, 10], [-17, 4, 5, 0], [-11, -3, 5, -3], [-13, -6, -1, -1], [23, -1, 3, 0], [2, -1, 4, -5], [-13, 4, -2, -3], [7, 2, -3, 1], [-18, 12, 3, 0], [-5, 4, 1, 0], [16, 4, 5, 4], [-3, -9, 1, -7], [-2, 2, 9, -2], [-11, 14, -5, 0], [22, -6, -1, -3], [-21, -3, 2, -3], [11, -9, 0, -11], [3, 1, 6, -7], [0, 8, 7, -3], [12, 9, 3, -5], [19, 2, -3, 5], [-13, 2, 4, 3], [-5, -5, 5, -4], [-12, 8, -5, 3], [-38, 4, 2, 4], [-11, -6, -8, 6], [-21, 6, -6, -2], [10, -10, 3, 6], [7, -1, -7, 1], [-3, -3, -5, -2], [-13, 5, -5, 5], [31, -6, 3, 2], [-11, 7, -7, 8], [-3, -11, -1, 0], [2, 2, -2, -3], [-26, -2, -2, 1], [-10, -5, 1, 1], [6, -3, 2, 2], [-6, -6, 12, -9], [11, 8, -3, 1], [-11, 14, -1, 13], [-27, 13, 1, 5], [13, 4, -7, 4], [-7, -7, 4, -8], [10, -3, 5, -4], [10, -14, 4, -2], [14, -10, -2, 1], [12, 6, 6, 6], [-12, -4, -3, -3], [20, -17, -3, -1], [-10, 2, 8, -4], [3, -1, -7, 4], [-15, -10, 3, -7], [12, 4, 6, -1], [0, 1, 1, -5], [12, 7, -3, 8], [1, 5, 4, -2], [13, -1, 9, -1], [38, -2, 4, -10], [-4, 17, -3, 1], [4, 8, 2, -9], [17, -1, -7, 2], [-7, -11, -4, 9], [20, -6, 6, -3], [5, 0, -11, 6], [10, 0, -2, -4], [23, 5, 4, -4], [21, -1, 3, -4], [25, -18, -4, -4], [24, -16, 2, 3], [-4, -15, 8, -11], [4, 2, -8, 10], [5, -12, -2, 1], [24, -10, 4, -8], [-2, -6, 5, -1], [23, -15, 3, 1], [-30, 5, 3, 0], [24, -2, 9, 1], [-4, 5, 4, -5], [2, -10, -6, 3], [18, 4, -4, 0], [20, 9, -2, 6], [-15, -13, 3, 6], [2, 0, -2, -12], [-38, 2, 3, -2], [-7, 16, -4, 13], [-24, 0, 4, 3], [-12, 1, 6, -5], [4, -7, 2, -6], [22, -7, 0, -6], [22, 0, 8, -1], [-3, 7, -3, -11], [12, 5, -2, -1], [-15, -3, 5, 2], [-20, -1, -10, 0], [-27, 5, 5, -4], [-8, -7, -4, 8], [-26, 10, -7, -5], [35, -2, 5, -13], [-37, 0, -9, 3], [-18, 5, -3, -2], [18, 0, 9, -8], [-12, 2, -9, 11], [11, 9, 12, -7], [-2, -5, 9, 2], [4, 7, -4, -1], [3, -8, 8, -9], [10, -1, 8, -9], [17, -2, 3, -6], [8, 4, 4, 1], [-38, 8, -4, 5], [12, -1, 4, 2], [-1, -11, -2, -6], [16, 12, -5, 7], [-10, 16, 5, 12], [16, 6, 4, -7], [16, -15, 5, -2], [21, 3, 7, -12], [-39, -4, -3, -4], [-9, 4, -5, -5], [-3, 1, -9, 8], [3, 10, -3, -7], [3, 7, -9, 4], [12, 6, -6, 3], [9, 3, -5, -2], [-17, 4, 9, -1], [10, 16, -5, 7], [12, -14, 6, -5], [-6, 5, -9, 5], [6, -5, 1, 8], [-20, 6, -10, 19], [12, -7, -4, 8], [2, 15, -14, 16], [-16, -8, 3, 8], [-16, -1, -2, 4], [29, -11, -5, -5], [15, 6, 1, 3], [-27, 4, 0, 1], [-3, 9, -3, 12], [10, 15, -2, -3], [12, -5, -14, 7], [-13, 5, -11, 7], [-20, 13, -12, 5], [7, 7, 13, -5], [17, -6, -5, -2], [-27, 12, 1, 0], [-10, -2, -4, 7], [-15, 10, 3, -3], [23, 0, 5, -17], [-2, -4, 5, -1], [27, 3, 0, 1], [26, -13, -3, -8], [32, 2, 10, 2], [-7, 18, -1, 1], [14, -15, -6, 1], [6, -3, -17, 8], [33, -4, 5, -5], [17, 13, -4, 3], [-14, 17, 6, 3], [36, -1, 6, -5], [6, 2, -1, 2], [16, 9, -5, 14], [0, 16, -6, 7], [17, 7, -5, 10], [38, -1, 12, -11], [-6, 10, -9, 2], [-21, 7, -14, 2], [-32, 3, -6, 3], [15, -9, 3, 11], [2, 14, 0, 13], [-34, 16, 4, 11], [-37, 1, -13, 7], [-32, -11, 2, -8], [35, -4, 7, 2], [-20, -8, -7, -11], [26, -15, 7, -2], [10, -4, 2, -2], [37, -24, -5, -4], [8, 8, 10, 7], [7, -13, 3, -5], [-36, 5, 3, 6], [24, -3, 6, -17], [2, 11, 8, -12], [-5, 0, -3, 6], [-5, 4, -10, 0], [-5, -11, -2, 4], [14, -1, -5, -1], [24, 11, -1, 7], [5, -4, 3, 4], [18, 10, -12, 13], [-2, 13, 0, 4], [-20, -4, -4, -6], [40, -6, 7, 6], [-13, 8, 1, -5], [-17, -8, -7, 4], [12, -17, -6, -11], [49, 6, -5, -1], [-1, -5, 7, -16], [-13, 9, 2, 7], [-23, 14, -1, 8], [16, 1, 5, -8], [17, -15, 5, -2], [37, -12, -7, -5], [17, 18, -7, 8], [19, -8, -5, 4], [-27, 0, -15, 15], [-30, 6, 8, 4], [3, -5, 3, -19], [15, -9, -11, 2], [35, 12, 3, 0], [-23, -3, -6, 9], [-29, 4, 5, -9], [-28, 6, -8, 21], [31, -10, -5, -2], [-23, -1, 5, -7], [18, 1, 4, -5], [-29, 9, -1, 9], [48, -8, 11, 3], [33, -4, 6, 8], [-19, 19, 6, -8], [-8, -16, 2, 1], [-24, -4, -2, -6], [-22, 8, 8, -1], [-22, 11, 9, 1], [-13, -3, -2, 18], [-40, 0, 4, 6], [-4, 2, -6, 7], [-44, 3, 5, 6], [4, -3, 11, 3], [37, -5, 3, 2], [39, -5, -1, -3], [41, -10, 4, -20], [-16, 11, -14, -2], [1, -3, 15, 0], [-5, 14, 1, -11], [-22, 14, -5, 3], [21, -17, 1, -2], [-16, 2, -3, -7], [21, -16, -6, 8], [0, 8, -18, 18], [-32, -13, -1, -5], [-12, 3, 6, 3], [-5, -3, -13, 13], [14, 17, 0, -2], [19, -5, 17, -10], [-32, 10, 2, 4], [23, 4, 7, 4], [-15, 24, 7, 2], [-14, 21, 1, 8], [-64, -2, 4, 1], [12, 12, 2, -5], [44, -20, 0, -2], [18, -12, 0, 12], [-11, 14, 3, 11], [-27, 4, 0, 16], [-49, -6, -2, -3], [28, 0, 0, -3], [35, 0, -9, 5], [19, 4, -10, 4], [-9, 3, 0, 7], [15, 7, -14, 6], [29, -11, -8, -5], [47, -6, 2, 6], [12, 16, 0, 1], [-7, 11, -2, -1], [-40, -6, -6, 1], [-10, 8, -8, 21], [5, -11, 5, 10], [-31, 7, 1, 6], [2, -9, 0, 6], [-3, 19, 7, -6], [-30, 5, -13, 17], [14, -6, 4, -9], [-26, -9, -8, -4], [46, -15, -4, -4], [-26, -19, 2, 3], [27, -15, -5, -1], [-29, -7, -7, 5], [31, -5, 7, -3], [-2, -6, 6, -11], [-22, 5, 1, 17], [-47, 2, 4, -6], [-13, -4, -2, 15], [17, 21, -7, 1], [-4, -6, -5, -12], [-47, 8, -5, 3], [-11, 20, -1, 13], [-25, -17, -2, 5], [-55, 17, 8, -4], [-18, 15, 2, 3], [9, -3, 24, -16], [-20, -4, -18, 13], [-45, -3, -3, 3], [-19, -5, 17, -15], [-6, 5, -21, 17], [10, -9, 2, -3], [13, -14, 4, 8], [14, -1, -3, 11], [-13, 17, -5, 14], [-35, 17, 2, 10], [-27, 9, -14, 3], [-51, 12, 3, 11], [-19, 20, 13, 2], [-49, -6, 4, -4], [5, 21, 5, 13], [30, -13, 9, 8], [-63, 5, -9, 6], [48, -2, -8, 7], [40, -7, -1, -19], [-5, 1, -7, -3], [-28, 3, -14, 22], [30, 8, 4, -6], [-14, 17, 11, -11]]
